@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -34,16 +34,24 @@
 #define SCAVENGE(ptr) FALSE
 #define FOUND(ptr1, ptr2) ptr2
 
-static Class *string_class;
-static int count_offset; 
-static int value_offset;
-static int offset_offset;
-
 static HashTable hash_table;
 
+static Class *string_class;
+static int value_offset;
+
+#ifdef SHARED_CHAR_BUFFERS
+static int count_offset; 
+static int offset_offset;
+#define STRING_LEN(string) INST_DATA(string, int, count_offset)
+#define STRING_OFFSET(string) INST_DATA(string, int, offset_offset)
+#else
+#define STRING_LEN(string) ARRAY_LEN(INST_DATA(string, Object*, value_offset))
+#define STRING_OFFSET(string) 0
+#endif
+
 int stringHash(Object *ptr) {
-    int len = INST_DATA(ptr, int, count_offset);
-    int offset = INST_DATA(ptr, int, offset_offset);
+    int len = STRING_LEN(ptr);
+    int offset = STRING_OFFSET(ptr);
     Object *array = INST_DATA(ptr, Object*, value_offset); 
     unsigned short *dpntr = ARRAY_DATA(array, unsigned short) + offset;
     int hash = 0;
@@ -55,14 +63,14 @@ int stringHash(Object *ptr) {
 }
 
 int stringComp(Object *ptr, Object *ptr2) {
-    int len = INST_DATA(ptr, int, count_offset);
-    int len2 = INST_DATA(ptr2, int, count_offset);
+    int len = STRING_LEN(ptr);
+    int len2 = STRING_LEN(ptr2);
 
     if(len == len2) {
+        int offset = STRING_OFFSET(ptr);
+        int offset2 = STRING_OFFSET(ptr2);
         Object *array = INST_DATA(ptr, Object*, value_offset);
         Object *array2 = INST_DATA(ptr2, Object*, value_offset);
-        int offset = INST_DATA(ptr, int, offset_offset);
-        int offset2 = INST_DATA(ptr2, int, offset_offset);
         unsigned short *src = ARRAY_DATA(array, unsigned short) + offset;
         unsigned short *dst = ARRAY_DATA(array2, unsigned short) + offset2;
 
@@ -88,7 +96,9 @@ Object *createString(char *utf8) {
     data = ARRAY_DATA(array, unsigned short);
     convertUtf8(utf8, data);
 
+#ifdef SHARED_CHAR_BUFFERS
     INST_DATA(ob, int, count_offset) = len; 
+#endif
     INST_DATA(ob, Object*, value_offset) = array; 
 
     return ob;
@@ -97,6 +107,9 @@ Object *createString(char *utf8) {
 Object *findInternedString(Object *string) {
     Object *interned;
 
+    if(string == NULL)
+        return NULL;
+
     /* Add if absent, no scavenge, locked */
     findHashEntry(hash_table, string, interned, TRUE, FALSE, TRUE);
 
@@ -138,7 +151,7 @@ void threadInternedStrings() {
 }
 
 char *String2Buff0(Object *string, char *buff, int len) {
-    int offset = INST_DATA(string, int, offset_offset);
+    int offset = STRING_OFFSET(string);
     Object *array = INST_DATA(string, Object*, value_offset);
     unsigned short *str = ARRAY_DATA(array, unsigned short) + offset;
     char *pntr;
@@ -151,43 +164,55 @@ char *String2Buff0(Object *string, char *buff, int len) {
 }
 
 char *String2Buff(Object *string, char *buff, int buff_len) {
-    int str_len = INST_DATA(string, int, count_offset);
+    int str_len = STRING_LEN(string);
     int len = buff_len-1 < str_len ? buff_len-1 : str_len;
 
     return String2Buff0(string, buff, len);
 }
 
 char *String2Cstr(Object *string) {
-    int len = INST_DATA(string, int, count_offset);
-    char *buff = (char *)sysMalloc(len + 1);
+    int len = STRING_LEN(string);
+    char *buff = sysMalloc(len + 1);
 
     return String2Buff0(string, buff, len);
 }
 
-void initialiseString() {
-    FieldBlock *count = NULL, *value = NULL, *offset = NULL;
+int initialiseString() {
+    FieldBlock *value;
 
     string_class = findSystemClass0(SYMBOL(java_lang_String));
+    if(string_class == NULL)
+        goto error;
+
+    value = findField(string_class, SYMBOL(value), SYMBOL(array_C));
+    if(value == NULL)
+        goto error;
+
     registerStaticClassRef(&string_class);
-
-    if(string_class != NULL) {
-        count = findField(string_class, SYMBOL(count), SYMBOL(I));
-        value = findField(string_class, SYMBOL(value), SYMBOL(array_C));
-        offset = findField(string_class, SYMBOL(offset), SYMBOL(I));
-    }
-
-    /* findField doesn't throw an exception... */
-    if((count == NULL) || (value == NULL) || (offset == NULL)) {
-        jam_fprintf(stderr, "Error initialising VM (initialiseString)\n");
-        exitVM(1);
-    }
-
-    count_offset = count->u.offset;
     value_offset = value->u.offset;
-    offset_offset = offset->u.offset;
+
+#ifdef SHARED_CHAR_BUFFERS
+    {
+        FieldBlock *count = findField(string_class, SYMBOL(count), SYMBOL(I));
+        FieldBlock *offset = findField(string_class, SYMBOL(offset),
+                                                     SYMBOL(I));
+
+        if(count == NULL || offset == NULL)
+            goto error;
+
+        count_offset = count->u.offset;
+        offset_offset = offset->u.offset;
+    }
+#endif
 
     /* Init hash table and create lock */
     initHashTable(hash_table, HASHTABSZE, TRUE);
+
+    return TRUE;
+
+error:
+    jam_fprintf(stderr, "Error initialising VM (initialiseString)\n");
+    return FALSE;
 }
 
 #ifndef NO_JNI
@@ -201,8 +226,11 @@ Object *createStringFromUnicode(unsigned short *unicode, int len) {
         unsigned short *data = ARRAY_DATA(array, unsigned short);
         memcpy(data, unicode, len * sizeof(unsigned short));
 
+#ifdef SHARED_CHAR_BUFFERS
         INST_DATA(ob, int, count_offset) = len; 
+#endif
         INST_DATA(ob, Object*, value_offset) = array; 
+
         return ob;
     }
     return NULL;
@@ -213,13 +241,13 @@ Object *getStringCharsArray(Object *string) {
 }
 
 unsigned short *getStringChars(Object *string) {
+    int offset = STRING_OFFSET(string);
     Object *array = INST_DATA(string, Object*, value_offset);
-    int offset = INST_DATA(string, int, offset_offset);
     return ARRAY_DATA(array, unsigned short) + offset;
 }
 
 int getStringLen(Object *string) {
-    return INST_DATA(string, int, count_offset);
+    return STRING_LEN(string);
 }
 
 int getStringUtf8Len(Object *string) {
@@ -233,7 +261,7 @@ char *StringRegion2Utf8(Object *string, int start, int len, char *utf8) {
 char *String2Utf8(Object *string) {
     int len = getStringLen(string);
     unsigned short *unicode = getStringChars(string);
-    char *utf8 = (char*)sysMalloc(utf8CharLen(unicode, len) + 1);
+    char *utf8 = sysMalloc(utf8CharLen(unicode, len) + 1);
 
     return unicode2Utf8(unicode, len, utf8);
 }
